/****************************************************************
 *                                                              *
 *                        I C A O   M a p  -                    *
 *                                                              *
 *             A n   A v i a t i o n   U t i l i t y            *
 *                                                              *
 *                                                              *
 *            Copyright (C) 1993, 1994  Martin Pauly            *
 *                                                              *
 *   This file may be freely distributed only if it includes    *
 *                the above copyright notice.                   *
 *                                                              *
 ****************************************************************/


/*  check world file for strange things */


#include <stdio.h>
#include "objects.h"



/* make compiler happy (and probably Oliver unhappy): */

int numscales, scale, scales, map, mapWidthcm, mapHeightcm,
    findvisibleobjects, map_drawline, grid_printunder, 
    grid_printleft, digi_input,
    digi_calibrate;


void initmapobjects()
{  /* do nothing */
}


extern int db_objects; 
extern OBJECT **objectlist;


/* prototypes */

int objects_init();
char *objecttypestring (int objecttype);


/* global variables to store objects */





extern FILE *yyin;


int main (int argc, char *argv[])
{
  int i, j, percentready=0, lastready = -1;
  FILE *inputfile;
  int recognized, objecttype;


  /* flags for command line optios */

  int cmd_nodelete = 0;   /* don't delete highways, airspace etc. */
  int cmd_statistic = 0;  /* display statistic about # of objects */
  double cmd_findclose = 0;  /* find objects that are close to each other */

  puts ("ICAO Map - Check World File Integrity    (c) 1994 Martin Pauly");

  objects_init();
  if (argc < 2)
    fprintf (stderr, "No input file specified!\n");
  else
    {
      for (i=1; i<argc; i++)
	{
	  if (argv[i][0] == '-')
	    {
	      recognized = 0;

	      if (!strcmp (argv[i], "-nodelete"))
		{
		  cmd_nodelete = 1;
		  recognized = 1;
		}

	      if (!strcmp (argv[i], "-statistic"))
		{
		  cmd_statistic = 1;
		  recognized = 1;
		}

	      if (!strcmp (argv[i], "-findclose"))
		{
		  sscanf (argv[++i], "%lf", &cmd_findclose);
		  recognized = 1;
		}

	      if (!recognized)
		fprintf (stderr, "Unknown option: %s\n", argv[i]);
	    }
	  else
	    {
	      inputfile = fopen (argv[i], "r");
	      if (inputfile)
		{
		  yyin = inputfile;
		  printf ("Reading objects from file %s\n", argv[i]);
		  yyparse();
		  fclose (yyin);
		}

	      else
		fprintf (stderr, "Unable to read input file %s\n",
			 argv[i]);
	    }
	}
      
      printf ("%d objects in memory.\n", db_objects);

      if (cmd_statistic)
	{
	  int counter[5000];
	  
	  for (i=0; i<5000; i++)
	    counter[i] = 0;

	  for (i=0; i<db_objects; i++)
	    counter[objectlist[i]->type]++;

	  printf ("\nType                          #objects\n");
	  printf ("--------------------------------------\n");
	  for (i=0; i<5000; i++)
	    if (counter[i])
	      printf ("%-30s%5d\n", objecttypestring(i),
		      counter[i]);
	}

      if (!cmd_nodelete)   /* delete area objects */
	{
	  i = 0;
	  while (i < db_objects)
	    {
	      objecttype = objectlist[i]->type;
	      if ((objecttype == O_RIVER) ||
		  (objecttype == O_ROAD ) ||
		  (objecttype == O_HIGHWAY ) ||
		  (objecttype == O_BORDER ) ||
		  (objecttype == O_STATE_BORDER ) ||
		  (objecttype == O_COUNTRY ) ||
		  (objecttype == O_ISLAND ) ||
		  (objecttype == O_ISOGONE ) ||
		  ((objectlist[i]->location.latitude == 0) && 
		   (objectlist[i]->location.longitude == 0)))
		{    /* delete this object */
		  objectlist[i] = objectlist[--db_objects];
		}
	      else
		i++;
	    }
	  printf ("%d objects remaining after deleting area objects.\n",
		  db_objects);
	}


      if (cmd_findclose)
	{
	  for (i=0; i<db_objects-1; i++)
	    for (j=i+1; j<db_objects; j++)
	      if (distance (objectlist[i]->location,
			    objectlist[j]->location) < cmd_findclose)
		{
		  printf ("%s (%s) and ",
			  objectlist[i]->name, 
			  objecttypestring (objectlist[i]->type));
		  printf ("%s (%s): %8.3lf NM distance\n",
			  objectlist[j]->name,
			  objecttypestring (objectlist[j]->type),
			  distance (objectlist[i]->location,
				    objectlist[j]->location));
		}
	}

    }

  return 0;
}


