%{
/****************************************************************
 *                                                              *
 *                        I C A O   M a p  -                    *
 *                                                              *
 *             A n   A v i a t i o n   U t i l i t y            *
 *                                                              *
 *                                                              *
 *            Copyright (C) 1993, 1994  Martin Pauly            *
 *                                                              *
 *   This file may be freely distributed only if it includes    *
 *                the above copyright notice.                   *
 *                                                              *
 ****************************************************************/

#include "objects.h"
#include "icao.h"
#include "setup.h"


LOCATION makelocation (long lat, long longi)
{
  LOCATION temp;

  temp.latitude = lat;
  temp.longitude = longi;
  return temp;
}

void map_setzero (long zero);

void add_for_load (char* name);

int currentline = 1;
char *filenamestring = NULL;

%}


%union {
	char *c;
        char s;
	float d;
	LOCATION l;
}


%token tSTRING
%token tFLOAT
%token tCHAR

%token tLOCATION
%token tLATITUDE
%token tLONGITUDE
%token tAT

%token tAIRSPACE
%token tFL
%token tGND
%token tMSL
%token tUNL

%token tSCALE tORIGIN tPARALLELS tZEROMERIDIAN
%token tSHOW_MAP
%token tDIGITIZER tCALIBRATE

%type <c> tSTRING
%type <d> tFLOAT
%type <s> tCHAR
%type <l> tLOCATION

%token tOUT

%token tAT tELEV tFREQUENCY tALIAS tRANGE tISOGONE
%token tINTLAIRPORT tAIRPORT tAIRPORT_CIV_MIL tAIRPORT_MIL tAIRFIELD tSPECIAL_AIRFIELD
%token tGLIDER_SITE tHANG_GLIDER_SITE tPARACHUTE_JUMPING_SITE tFREE_BALLON_SITE
%token tHELIPORT tHELIPORT_AMB tRIVER tHIGHWAY tROAD tLAKE

%token tBORDER, tSTATE_BORDER
%token tCOUNTRY, tSTATE
%token tISLAND

%token tCTR, tCVFR

%token tVOR tVOR_DME tVORTAC tTACAN tNDB tMARKER_BEACON tBASIC_RADIO_FACILITY 

%token tOBSTRUCTION tGROUP_OBSTRUCTION tFIRED_OBSTRUCTION tFIRED_GROUP_OBSTRUCTION

%token tREPORTING_POINT, tWAYPOINT, tVILLAGE, tTOWN

%token tRUNWAY tGRAS tASPHALT tCONCRETE

%token tSET, tUNIT, tDISTANCE, tSPEED, tALT, tKM, tNM, tKMH, tKNOTS, tMPS, tM, tFT
%token tPROJECTION, tLAMBERT, tMERCATOR, tVISIBLE
%token tALGORITHM, tPAULY, tFAI, tLOAD, tMAP

%%

input:	/* empty */
	| input line
;


config:
	tSET tUNIT tDISTANCE tKM      { unit_distance = km; }
	| tSET tUNIT tDISTANCE tNM    { unit_distance = nm; }
	| tSET tUNIT tSPEED tKMH      { unit_speed = kmh; }
	| tSET tUNIT tSPEED tKNOTS    { unit_speed = kts; }
	| tSET tUNIT tSPEED tMPS      { unit_speed = mps; }
	| tSET tUNIT tALT tM          { unit_altitude = mtr; }
	| tSET tUNIT tALT tFT         { unit_altitude = ft; }
	| tSET tPROJECTION tLAMBERT   { map_projection = 0; }	
	| tSET tPROJECTION tMERCATOR  { map_projection = 1; }
	| tSET tVISIBLE tSTRING	      { set_visiblestring ($<c>3); }
	| tSET tALGORITHM tDISTANCE tPAULY { config_distance_func = pauly; }
	| tSET tALGORITHM tDISTANCE tFAI   { config_distance_func = FAI; }


line:	'\n'
	| loc object qualities
	| object qualities
	| loc elev
	| command
;

command:
	tOUT tSTRING		{ puts ($<c>2); }
	| tSET tSCALE tFLOAT ':' tFLOAT	{ map_setscale ($<d>5); }
	| tSET tORIGIN location	{ map_setorigin ($<l>3); }
	| tSET tPARALLELS tLATITUDE tLATITUDE  { map_setparallels ($<d>3, $<d>4); }
	| tSHOW_MAP	{ map_display(); }
	| tDIGITIZER	{ digi_input(); }
	| tCALIBRATE location location	{ digi_calibrate($<l>2, $<l>3); }
	| tSET tZEROMERIDIAN tLONGITUDE	{ map_setzero ((long) $<d>3); }
	| config
	| tLOAD tMAP tSTRING            { add_for_load ($<c>3); }
;

location:
	tLATITUDE tLONGITUDE {$<l>$ = makelocation((long)$<d>1, (long)$<d>2);}

;

limit:
	tFL tFLOAT	{ $<d>$ = 10 * $<d>2; }
	| tFLOAT tGND	{ $<d>$ = 10 * $<d>2 + 1; }
	| tGND		{ $<d>$ = 1; }
	| tFLOAT tMSL	{ $<d>$ = 10 * $<d>2 + 2; }
	| tUNL		{ $<d>$ = 3; }
;

airspace:
	tAIRSPACE tSTRING tSTRING limit limit pointlist { new_airspace ($<c>2, $<c>3, $<d>4, $<d>5); }
;

object:
	tINTLAIRPORT tSTRING pointlist      { new_object (O_INTLAIRPORT, $<c>2); }
	| airspace
	| tAIRPORT tSTRING pointlist        { new_object (O_AIRPORT, $<c>2); }
	| tAIRPORT_CIV_MIL         tSTRING  { new_object (O_AIRPORT_CIV_MIL, $<c>2); }
	| tAIRPORT_MIL             tSTRING  { new_object (O_AIRPORT_MIL, $<c>2); }
	| tAIRFIELD                tSTRING  { new_object (O_AIRFIELD, $<c>2); }
	| tSPECIAL_AIRFIELD        tSTRING  { new_object (O_SPECIAL_AIRFIELD, $<c>2); }
	| tHELIPORT                tSTRING  { new_object (O_HELIPORT, $<c>2); }
	| tHELIPORT_AMB            tSTRING  { new_object (O_HELIPORT_AMB, $<c>2); }
	| tGLIDER_SITE             tSTRING  { new_object (O_GLIDER_SITE, $<c>2); }
	| tPARACHUTE_JUMPING_SITE  tSTRING  { new_object (O_PARACHUTE_JUMPING_SITE, $<c>2); }
	| tFREE_BALLON_SITE        tSTRING  { new_object (O_FREE_BALLON_SITE, $<c>2); }
	| tVOR                     tSTRING  { new_object (O_VOR, $<c>2); }
	| tVOR_DME                 tSTRING  { new_object (O_VOR_DME, $<c>2); }
	| tVORTAC                  tSTRING  { new_object (O_VORTAC, $<c>2); }
	| tTACAN                   tSTRING  { new_object (O_TACAN, $<c>2); }
	| tNDB                     tSTRING  { new_object (O_NDB, $<c>2); }
	| tMARKER_BEACON           tSTRING  { new_object (O_MARKER_BEACON, $<c>2); }
	| tBASIC_RADIO_FACILITY    tSTRING  { new_object (O_BASIC_RADIO_FACILITY, $<c>2); }
	| tOBSTRUCTION             tSTRING  { new_object (O_OBSTRUCTION, $<c>2); }
	| tGROUP_OBSTRUCTION       tSTRING  { new_object (O_GROUP_OBSTRUCTION, $<c>2); }
	| tFIRED_OBSTRUCTION       tSTRING  { new_object (O_FIRED_OBSTRUCTION, $<c>2); }
	| tFIRED_GROUP_OBSTRUCTION tSTRING  { new_object (O_FIRED_GROUP_OBSTRUCTION, $<c>2); }
	| tREPORTING_POINT         tSTRING  { new_object (O_REPORTING_POINT, $<c>2); }
	| tRIVER tSTRING pointlist	    { new_object (O_RIVER, $<c>2); }
	| tRIVER pointlist		    { new_object (O_RIVER, ""); }
	| tLAKE tSTRING pointlist	    { new_object (O_LAKE, $<c>2); }
	| tLAKE pointlist	    	    { new_object (O_LAKE, ""); }
	| tISOGONE tFLOAT pointlist	    { new_elev ($<d>2); 
					      new_object (O_ISOGONE, ""); }
	| tHIGHWAY pointlist		    { new_object (O_HIGHWAY, ""); }
	| tHIGHWAY tSTRING pointlist	    { new_object (O_HIGHWAY, $<c>2); }
	| tROAD pointlist		    { new_object (O_ROAD, ""); }
	| tVILLAGE tSTRING		    { new_object (O_VILLAGE, $<c>2); }
	| tTOWN tSTRING pointlist	    { new_object (O_TOWN, $<c>2); }
	| tTOWN pointlist		    { new_object (O_TOWN, ""); }
	| tWAYPOINT tSTRING		    { new_object (O_WAYPOINT, $<c>2); }
	| tCTR tSTRING pointlist	    { new_object (O_CTR, $<c>2); }
	| tCVFR tSTRING pointlist	    { new_object (O_CVFR, $<c>2); }
        | tBORDER pointlist	            { new_object (O_BORDER, ""); }
	| tSTATE_BORDER pointlist	    { new_object (O_STATE_BORDER, ""); }
	| tCOUNTRY tSTRING        	    { new_object (O_COUNTRY, $<c>2); }
	| tSTATE tSTRING        	    { new_object (O_STATE, $<c>2); }
	| tISLAND tSTRING pointlist	    { new_object (O_ISLAND, $<c>2); }
	| tISLAND pointlist	            { new_object (O_ISLAND, ""); }
;

pointlist:	/* empty */
	| pointlist location		{ add_point ($<l>2); }

;

qualities:	/* empty */
	| qualities quality

;


elev:	tELEV tFLOAT     { new_elev ($<d>2); }
;

loc:	tAT location     { new_location ($<l>2); }

quality:
	tFREQUENCY tFLOAT { set_frequency ($<d>2); }
	| tRUNWAY tFLOAT tFLOAT runwaytype { add_runway ($<d>2, $<d>3, $<s>4); }
	| tALIAS tSTRING { set_alias ($<c>2); }
	| tRANGE tFLOAT { set_range ($<d>2); }
;

runwaytype:
	tASPHALT		{ $<s>$ = 'A'; }
	| tCONCRETE		{ $<s>$ = 'C'; }
	| tGRAS			{ $<s>$ = 'G'; }
;




%%


yyerror (char *s)
{
  printf ("%s processing line %d of file %s\n", s, currentline, filenamestring);
}
