/****************************************************************
 *                                                              *
 *                        I C A O   M a p  -                    *
 *                                                              *
 *             A n   A v i a t i o n   U t i l i t y            *
 *                                                              *
 *                                                              *
 *            Copyright (C) 1993, 1994  Martin Pauly            *
 *                                                              *
 *   This file may be freely distributed only if it includes    *
 *                the above copyright notice.                   *
 *                                                              *
 *       Special thanks to Thomas Petrick for dealing with      *
 *                     my printing routines                     *
 *                                                              *
 ****************************************************************/


#include "graph_primi.h"
#include "objects.h"
#include <stdio.h>
#include <Xm/XmP.h>
#include <X11/Shell.h>
#include <Xm/Protocols.h>
#include <Xm/BulletinB.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>
#include "busy.h"
#include "gpssim.h"


extern int media;     /* SCREEN, POSTSCRIPT or COLORPS are defined in graph_primi.c */
extern int mapWidth, mapHeight;
extern double mapWidthcm, mapHeightcm;
extern LOCATION map_origin;
extern long map_scale;
extern Widget toplevel;


FILE *ps;

int originalWidth, originalHeight;
long originalScale;
double originalWidthcm, originalHeightcm;
double   paperheight, paperwidth;
int pointsh,pointsw;
LOCATION originalorigin;
Widget shell, printfile;
static int orientation = 1;   /* default: landscape */
static int size = 0;          /* default: a4 */
static int device = 0;        /* default: b/w Printer */
static char printfilename[128] = "map.ps";


#define FROM  0
#define VIA1  1
#define VIA2  2
#define VIA3  3
#define DEST  4

/* page sizes */
#define DIN_A4_WIDTH 17.6
#define DIN_A3_WIDTH 26.7
#define DIN_A4_HEIGHT 25.7
#define DIN_A3_HEIGHT 37.4

/* points per page */
#define DIN_A4_POINTS_H 1400
#define DIN_A4_POINTS_W 1000
#define DIN_A3_POINTS_H 1960
#define DIN_A3_POINTS_W 1400


/* current route to be drawn */

extern OBJECT *currentroute[];
extern int drawroutenumpoints;
extern LOCATION drawroutepoints[100];

/*
** EXPORT: for external use
*/
void  print_dialog();  /* open dialog */
void  Popdown_print(); /* close dialog */

int   print_IsPoppedUp = False;  /* open dialog only once */

Widget print = NULL;   /* DialogShell-Widget */

Widget pr_landscapeTB = NULL;
Widget pr_portraitTB = NULL;
Widget pr_bwTB = NULL;
Widget pr_colorTB = NULL;
Widget pr_a4TB = NULL;
Widget pr_a3TB = NULL;
Widget printfile = NULL;
Widget pr_cancel_PB = NULL;
Widget pr_prmap_PB = NULL;
Widget pr_prroute_PB = NULL;

/* prototypes */

void psheader(int landscape);
void pstrailer();
void coord_drawgrid (int printlabels);

void printorientCB (Widget w, XtPointer client_data, XmToggleButtonCallbackStruct *cbs)
{
  orientation = (int) client_data;
}

void sizeCB (Widget w, XtPointer client_data, XmToggleButtonCallbackStruct *cbs)
{
  size=(int) client_data;
}

void deviceCB (Widget w, XtPointer client_data, XmToggleButtonCallbackStruct *cbs)
{
  device=(int) client_data;
}

void printCB (Widget w, XtPointer client_data, XmPushButtonCallbackStruct *cbs)
{
  XmString temp;
  Widget   warning;
  Arg      args[10];
  char     *tempstr;
  int      printroute = ((int) client_data) == 2, i;
  LOCATION topleft, botright, a, b;
  double   heightNM, widthNM, scalehor, scalever;
  int      x1, y1, x2, y2;


  if (client_data)
    {
      /* read filename from dialog box */

      XtVaGetValues (printfile,
		     XmNvalue, &tempstr,
		     NULL);
             
      paperheight= (size==0) ? DIN_A4_HEIGHT : DIN_A3_HEIGHT;
      pointsh = (size==0) ? DIN_A4_POINTS_H : DIN_A3_POINTS_H;
      paperwidth = (size==0) ? DIN_A4_WIDTH : DIN_A3_WIDTH;       
      pointsw = (size==0) ? DIN_A4_POINTS_W : DIN_A3_POINTS_W;

      strcpy (printfilename, tempstr);
      
      media = (device == 0) ? POSTSCRIPT : COLORPS;
      
      if (ps = fopen (printfilename, "w"))
	{
	  originalWidth = mapWidth;
	  originalHeight = mapHeight;
	  originalWidthcm = mapWidthcm;
	  originalHeightcm = mapHeightcm;
	  originalScale = map_scale;
	  originalorigin = map_origin;
	  
	  /* if route is to be printed, find out best orientation and scale */
	  
	  if (printroute)
	    {
	      /* check if there is no detailed route set up yet */
	      
	      if (drawroutenumpoints == 0)
		{
		  if (currentroute[FROM])
		    drawroutepoints[drawroutenumpoints++] = currentroute[FROM]->location;
		  if (currentroute[VIA1])
		    drawroutepoints[drawroutenumpoints++] = currentroute[VIA1]->location;
		  if (currentroute[VIA2])
		    drawroutepoints[drawroutenumpoints++] = currentroute[VIA2]->location;
		  if (currentroute[VIA3])
		    drawroutepoints[drawroutenumpoints++] = currentroute[VIA3]->location;
		  if (currentroute[DEST])
		    drawroutepoints[drawroutenumpoints++] = currentroute[DEST]->location;
		}
	      
	      /* check if we have enough waypoints (min. 2 needed) */
	      
	      if (drawroutenumpoints >= 2)
		{
		  /* find extension of area to be printed */
		  
		  topleft  = drawroutepoints[0];
		  botright = drawroutepoints[0];
		  
		  for (i=1; i<drawroutenumpoints; i++)
		    {
		      if (drawroutepoints[i].longitude < topleft.longitude)
			topleft.longitude = drawroutepoints[i].longitude;
		      if (drawroutepoints[i].latitude < topleft.latitude)
			topleft.latitude = drawroutepoints[i].latitude;
		      if (drawroutepoints[i].longitude > botright.longitude)
			botright.longitude = drawroutepoints[i].longitude;
		      if (drawroutepoints[i].latitude > botright.latitude)
			botright.latitude = drawroutepoints[i].latitude;
		    }
		  
		  /* calc width and height of map in NM */
		  
		  a.longitude = b.longitude = topleft.longitude;
		  a.latitude = topleft.latitude;
		  b.latitude = botright.latitude;
		  heightNM = distance (a, b);
		  
		  b.longitude = botright.longitude;
		  b.latitude  = topleft.latitude;
		  widthNM = distance (a, b);
		  
		  /* suggest either landscape (1) or portrait (0) */
		  
		  orientation = (widthNM > heightNM) ? 1 : 0;
		  
		  
		  /* now we need to find a scale that makes everything fit: */
		  
		  if (orientation)     /* landscape */
		    {
		      scalehor = widthNM * 185200 / paperheight;
		      scalever = heightNM * 185200 / paperwidth;
		    }
		  else
		    {
		      scalehor = widthNM * 185200 / paperwidth;
		      scalever = heightNM * 185200 / paperheight;
		    }
		  
		  map_scale = (long) ((scalehor > scalever) ? scalehor : scalever);
		  map_scale = (long) (map_scale * 1.1);
		  
		  /* last but not least: choose map origin so that everything will fit */
		  
		  map_origin.longitude = (long) ((topleft.longitude + botright.longitude) / 2);
		  map_origin.latitude  = (long) ((topleft.latitude  + botright.latitude)  / 2);
		  
		  
		}
	    }
	  
	  /* from here, it dowsn't matter any more whether we are drawing a
	     simple map or one that contains a route */
	 	  
	  psheader(orientation);
	  
	  map_initcoord();
	  
	  fprintf (ps, "0.2 setlinewidth\n");
	  fprintf (ps, "[] 0 setdash\n");
	  coord_drawgrid (1);         /* draw coordinate grid */
	  
	  drawvisibleobjects (0);   /* objects other than CTRs... */
	  drawvisibleobjects (1);   /* and CTRs */
	  
	  /* draw route lines if required */

	  if (log_numpoints) {
	    gps_draw_log();
	  }
	  
	  if (printroute && (drawroutenumpoints >= 2))
	    {
	      gp_setlinestyle (1, SOLID);
	      gp_setcolor (BLACK);
	      
	      internal2window (drawroutepoints[0], &x1, &y1);
	      
	      for (i=1; i<drawroutenumpoints; i++)
		{
		  internal2window (drawroutepoints[i], &x2, &y2);
		  gp_drawline (x1, y1, x2, y2);
		  
		  x1 = x2;   y1 = y2;
		}
	    }
	  
	  pstrailer();
	  
	  map_initcoord();
	  
	  fclose (ps);
	}
      else
	{
	  temp = XmStringCreateSimple ("Could not print to specified file!");
	  
	  i = 0;
	  XtSetArg (args[i], XmNtitle,   "Warning!");     i++;
	  warning = (Widget) XmCreateInformationDialog (toplevel, "warning", args, i);
	  
	  XtVaSetValues (warning, 
			 XmNmessageString, temp,
			 NULL);
	  XtUnmanageChild ((Widget) XmMessageBoxGetChild (warning, XmDIALOG_HELP_BUTTON));
	  
	  XtManageChild (warning);
	  
	  XmStringFree (tempstr);
	}
      
      media = SCREEN;
      
    }
    Popdown_print();
}
/**********************************************************************
**
*/
void print_dialog()
{
   extern Display *display;
   extern Screen  *screen;
   Arg args[10];
   Cardinal argcount;
   XmString xms[1];
   XColor def, exact;
   XFontStruct *xfontstruct;

   Widget label1 = NULL;
   Widget rowcolumn = NULL;
   Widget label18 = NULL;
   Widget rowcolumn0 = NULL;
   Widget label180 = NULL;
   Widget rowcolumn1 = NULL;
   Widget label181 = NULL;
   Widget rowcolumn14 = NULL;
   Widget label117 = NULL;
   Widget separator1 = NULL;
   Widget bulletinb = NULL;
   
   if( print == NULL ) {
     
     /*
     ** Codegeneration for the Widget print
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
       XtSetArg( args[argcount], XmNx, (Position) 200 ); argcount++; 
       XtSetArg( args[argcount], XmNy, (Position) 190 ); argcount++; 
       XtSetArg( args[argcount], XmNwidth, (Dimension) 390 ); argcount++;
       XtSetArg( args[argcount], XmNheight, (Dimension) 228 ); argcount++;
       XtSetArg( args[argcount], XmNdefaultPosition, False ); argcount++;
       XtSetArg( args[argcount], XmNautoUnmanage, False ); argcount++;
       xms[0] = (XmString) XmStringCreateLtoR( "ICAO Map Print", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNdialogTitle, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNnoResize, True ); argcount++;
     print = XmCreateFormDialog( toplevel, "print", args, argcount );
     XmStringFree( xms[0] );
     
     
     /*
     ** Codegeneration for the Widget label1
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "Choose Print Options:", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNx, (Position) 0 ); argcount++;
       XtSetArg( args[argcount], XmNy, (Position) 0 ); argcount++;
     label1 = XmCreateLabel( print, "label1", args, argcount );
     XtManageChild( label1 );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget rowcolumn
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNradioBehavior, True ); argcount++;
       XtSetArg( args[argcount], XmNorientation, XmHORIZONTAL ); argcount++;
       XtSetArg( args[argcount], XmNpacking, XmPACK_COLUMN ); argcount++;
       XtSetArg( args[argcount], XmNtearOffModel, XmTEAR_OFF_ENABLED ); argcount++;
     rowcolumn = XmCreateRowColumn( print, "rowcolumn", args, argcount );
     XtManageChild( rowcolumn );
     
     /*
     ** Codegeneration for the Widget label18
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "Orientation:", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
     label18 = XmCreateLabel( rowcolumn, "label18", args, argcount );
     XtManageChild( label18 );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget pr_landscapeTB
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "Landscape", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNset, True ); argcount++;
     pr_landscapeTB = XmCreateToggleButton( rowcolumn, "pr_landscapeTB", args, argcount );
     XtAddCallback (pr_landscapeTB, XmNvalueChangedCallback, (XtCallbackProc) printorientCB,
		 (XtPointer) 1);
     XtManageChild( pr_landscapeTB );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget pr_portraitTB
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "Portrait", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
     pr_portraitTB = XmCreateToggleButton( rowcolumn, "pr_portraitTB", args, argcount );
     XtAddCallback (pr_portraitTB, XmNvalueChangedCallback, (XtCallbackProc) printorientCB,
		 (XtPointer) 0);
     XtManageChild( pr_portraitTB );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget rowcolumn0
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNradioBehavior, True ); argcount++;
       XtSetArg( args[argcount], XmNorientation, XmHORIZONTAL ); argcount++;
       XtSetArg( args[argcount], XmNpacking, XmPACK_COLUMN ); argcount++;
       XtSetArg( args[argcount], XmNtearOffModel, XmTEAR_OFF_ENABLED ); argcount++;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
     rowcolumn0 = XmCreateRowColumn( print, "rowcolumn0", args, argcount );
     XtManageChild( rowcolumn0 );
     
     /*
     ** Codegeneration for the Widget label180
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "Device:     ", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
     label180 = XmCreateLabel( rowcolumn0, "label180", args, argcount );
     XtManageChild( label180 );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget pr_bwTB
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "b/w", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
       XtSetArg( args[argcount], XmNset, True ); argcount++;
     pr_bwTB = XmCreateToggleButton( rowcolumn0, "pr_bwTB", args, argcount );
      XtAddCallback (pr_bwTB, XmNvalueChangedCallback, (XtCallbackProc) deviceCB, (XtPointer) 0);
     XtManageChild( pr_bwTB );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget pr_colorTB
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "Color", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
     pr_colorTB = XmCreateToggleButton( rowcolumn0, "pr_colorTB", args, argcount );
      XtAddCallback (pr_colorTB, XmNvalueChangedCallback, (XtCallbackProc) deviceCB, (XtPointer) 1);
     XtManageChild( pr_colorTB );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget rowcolumn1
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNradioBehavior, True ); argcount++;
       XtSetArg( args[argcount], XmNorientation, XmHORIZONTAL ); argcount++;
       XtSetArg( args[argcount], XmNpacking, XmPACK_COLUMN ); argcount++;
       XtSetArg( args[argcount], XmNtearOffModel, XmTEAR_OFF_ENABLED ); argcount++;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
     rowcolumn1 = XmCreateRowColumn( print, "rowcolumn1", args, argcount );
     XtManageChild( rowcolumn1 );
     
     /*
     ** Codegeneration for the Widget label181
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "Paper Sheet:", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
     label181 = XmCreateLabel( rowcolumn1, "label181", args, argcount );
     XtManageChild( label181 );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget pr_a4TB
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "A4", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
       XtSetArg( args[argcount], XmNset, True ); argcount++;
     pr_a4TB = XmCreateToggleButton( rowcolumn1, "pr_a4TB", args, argcount );
        XtAddCallback (pr_a4TB, XmNvalueChangedCallback, (XtCallbackProc) sizeCB,(XtPointer) 0);
     XtManageChild( pr_a4TB );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget pr_a3TB
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "A3", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
     pr_a3TB = XmCreateToggleButton( rowcolumn1, "pr_a3TB", args, argcount );
       XtAddCallback (pr_a3TB, XmNvalueChangedCallback, (XtCallbackProc) sizeCB,(XtPointer) 1);
     XtManageChild( pr_a3TB );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget rowcolumn14
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNorientation, XmHORIZONTAL ); argcount++;
       XtSetArg( args[argcount], XmNtearOffModel, XmTEAR_OFF_ENABLED ); argcount++;
     rowcolumn14 = XmCreateRowColumn( print, "rowcolumn14", args, argcount );
     XtManageChild( rowcolumn14 );
     
     /*
     ** Codegeneration for the Widget label117
     */
       argcount = 0;
       xms[0] = (XmString) XmStringCreateLtoR( "Output to:  ", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
     label117 = XmCreateLabel( rowcolumn14, "label117", args, argcount );
     XtManageChild( label117 );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget printfile
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNvalue, "map.ps" ); argcount++;
     printfile = XmCreateText( rowcolumn14, "printfile", args, argcount );
     XtManageChild( printfile );
     
     /*
     ** Codegeneration for the Widget separator1
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNx, (Position) 0 ); argcount++;
       XtSetArg( args[argcount], XmNy, (Position) 0 ); argcount++;
     separator1 = XmCreateSeparator( print, "separator1", args, argcount );
     XtManageChild( separator1 );
     
     /*
     ** Codegeneration for the Widget bulletinb
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNunitType, XmPIXELS ); argcount++;
       XtSetArg( args[argcount], XmNwidth, (Dimension) 404 ); argcount++;
       XtSetArg( args[argcount], XmNheight, (Dimension) 316 ); argcount++;
     bulletinb = XmCreateBulletinBoard( print, "bulletinb", args, argcount );
     XtManageChild( bulletinb );
     
     /*
     ** Codegeneration for the Widget pr_cancel_PB
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNwidth, (Dimension) 104 ); argcount++;
       XtSetArg( args[argcount], XmNheight, (Dimension) 35 ); argcount++;
       xms[0] = (XmString) XmStringCreateLtoR( "Cancel", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNx, (Position) 10 ); argcount++;
       XtSetArg( args[argcount], XmNy, (Position) 24 ); argcount++;
     pr_cancel_PB = XmCreatePushButton( bulletinb, "pr_cancel_PB", args, argcount );
      XtAddCallback(pr_cancel_PB, XmNactivateCallback, (XtCallbackProc) printCB, (XtPointer) 0L );
     XtManageChild( pr_cancel_PB );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget pr_prmap_PB
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNwidth, (Dimension) 108 ); argcount++;
       XtSetArg( args[argcount], XmNheight, (Dimension) 35 ); argcount++;
       xms[0] = (XmString) XmStringCreateLtoR( "Print Map", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNx, (Position) 140 ); argcount++;
       XtSetArg( args[argcount], XmNy, (Position) 24 ); argcount++;
     pr_prmap_PB = XmCreatePushButton( bulletinb, "pr_prmap_PB", args, argcount );
      XtAddCallback( pr_prmap_PB, XmNactivateCallback, (XtCallbackProc) printCB, (XtPointer) 1L );
     XtManageChild( pr_prmap_PB );
     XmStringFree( xms[0] );
     
     /*
     ** Codegeneration for the Widget pr_prroute_PB
     */
       argcount = 0;
       XtSetArg( args[argcount], XmNwidth, (Dimension) 108 ); argcount++;
       XtSetArg( args[argcount], XmNheight, (Dimension) 35 ); argcount++;
       xms[0] = (XmString) XmStringCreateLtoR( "Print Route", XmSTRING_DEFAULT_CHARSET );
       XtSetArg( args[argcount], XmNlabelString, xms[0] ); argcount++;
       XtSetArg( args[argcount], XmNx, (Position) 272 ); argcount++;
       XtSetArg( args[argcount], XmNy, (Position) 24 ); argcount++;
     pr_prroute_PB = XmCreatePushButton( bulletinb, "pr_prroute_PB", args, argcount );
      XtAddCallback( pr_prroute_PB, XmNactivateCallback, (XtCallbackProc) printCB, (XtPointer) 2L );
     XtManageChild( pr_prroute_PB );
     XmStringFree( xms[0] );
     
     argcount = 0;
     XtSetArg( args[argcount], XmNtopAttachment, XmATTACH_FORM ); argcount++;
     XtSetArg( args[argcount], XmNbottomAttachment, XmATTACH_NONE ); argcount++;
     XtSetArg( args[argcount], XmNleftAttachment, XmATTACH_FORM ); argcount++;
     XtSetArg( args[argcount], XmNrightAttachment, XmATTACH_FORM ); argcount++;
     XtSetValues( label1, args, argcount );

     argcount = 0;
     XtSetArg( args[argcount], XmNtopWidget, label1 ); argcount++;
     XtSetArg( args[argcount], XmNtopAttachment, XmATTACH_WIDGET ); argcount++;
     XtSetArg( args[argcount], XmNbottomAttachment, XmATTACH_NONE ); argcount++;
     XtSetArg( args[argcount], XmNleftAttachment, XmATTACH_FORM ); argcount++;
     XtSetArg( args[argcount], XmNrightAttachment, XmATTACH_FORM ); argcount++;
     XtSetValues( rowcolumn, args, argcount );

     argcount = 0;
     XtSetArg( args[argcount], XmNtopWidget, rowcolumn ); argcount++;
     XtSetArg( args[argcount], XmNtopAttachment, XmATTACH_WIDGET ); argcount++;
     XtSetArg( args[argcount], XmNbottomAttachment, XmATTACH_NONE ); argcount++;
     XtSetArg( args[argcount], XmNleftAttachment, XmATTACH_FORM ); argcount++;
     XtSetArg( args[argcount], XmNrightAttachment, XmATTACH_FORM ); argcount++;
     XtSetValues( rowcolumn0, args, argcount );

     argcount = 0;
     XtSetArg( args[argcount], XmNtopWidget, rowcolumn0 ); argcount++;
     XtSetArg( args[argcount], XmNtopAttachment, XmATTACH_WIDGET ); argcount++;
     XtSetArg( args[argcount], XmNbottomAttachment, XmATTACH_NONE ); argcount++;
     XtSetArg( args[argcount], XmNleftAttachment, XmATTACH_FORM ); argcount++;
     XtSetArg( args[argcount], XmNrightAttachment, XmATTACH_FORM ); argcount++;
     XtSetValues( rowcolumn1, args, argcount );

     argcount = 0;
     XtSetArg( args[argcount], XmNtopWidget, rowcolumn1 ); argcount++;
     XtSetArg( args[argcount], XmNtopAttachment, XmATTACH_WIDGET ); argcount++;
     XtSetArg( args[argcount], XmNbottomAttachment, XmATTACH_NONE ); argcount++;
     XtSetArg( args[argcount], XmNleftAttachment, XmATTACH_FORM ); argcount++;
     XtSetArg( args[argcount], XmNrightAttachment, XmATTACH_FORM ); argcount++;
     XtSetValues( rowcolumn14, args, argcount );

     argcount = 0;
     XtSetArg( args[argcount], XmNtopWidget, rowcolumn14 ); argcount++;
     XtSetArg( args[argcount], XmNtopAttachment, XmATTACH_WIDGET ); argcount++;
     XtSetArg( args[argcount], XmNbottomAttachment, XmATTACH_NONE ); argcount++;
     XtSetArg( args[argcount], XmNleftAttachment, XmATTACH_FORM ); argcount++;
     XtSetArg( args[argcount], XmNrightAttachment, XmATTACH_FORM ); argcount++;
     XtSetValues( separator1, args, argcount );

     argcount = 0;
     XtSetArg( args[argcount], XmNtopWidget, separator1 ); argcount++;
     XtSetArg( args[argcount], XmNtopAttachment, XmATTACH_WIDGET ); argcount++;
     XtSetArg( args[argcount], XmNbottomAttachment, XmATTACH_NONE ); argcount++;
     XtSetArg( args[argcount], XmNleftAttachment, XmATTACH_POSITION ); argcount++;
     XtSetArg( args[argcount], XmNrightAttachment, XmATTACH_NONE ); argcount++;
     XtSetArg( args[argcount], XmNleftPosition, 0 ); argcount++;
     XtSetValues( bulletinb, args, argcount );

   }

   XtManageChild( print );
   print_IsPoppedUp = True;
   
   
   argcount = 0;
   XtSetArg( args[argcount], XmNshellUnitType, XmPIXELS ); argcount++;
   XtSetValues( XtParent( print ), args, argcount);
}

/**********************************************************************
**
*/
void Popdown_print()
{
   if( print_IsPoppedUp || ( print && XtIsManaged( print ) ) ) {
     XtUnmanageChild( print );
     print_IsPoppedUp = False;
   }
}

void psheader(int landscape)
{
  fprintf (ps, "%%! ICAO map output\n\n");
  fprintf (ps, "40 40 translate\n");
  fprintf (ps, "0.5 0.5 scale\n");

  if (landscape)
    {
      mapWidth = pointsh;
      mapHeight = pointsw;
      mapWidthcm = paperheight;
      mapHeightcm = paperwidth;

      fprintf (ps, "270 rotate\n");
      fprintf (ps, "-%d %d translate\n",pointsh,pointsw);
      fprintf (ps, "1 -1 scale\n");
    }
  else
    {
      mapWidth = pointsw;
      mapHeight = pointsh;
      mapWidthcm = paperwidth;
      mapHeightcm = paperheight;

      fprintf (ps, "0 %d translate\n",pointsh);
      fprintf (ps, "1 -1 scale\n");
    }


  fprintf (ps, "newpath\n");
  fprintf (ps, "0 0 moveto\n");
  fprintf (ps, "0 %d lineto\n", mapHeight);
  fprintf (ps, "%d %d lineto\n", mapWidth, mapHeight);
  fprintf (ps, "%d 0 lineto\n", mapWidth);
  fprintf (ps, "0 0 lineto\n");
  fprintf (ps, "closepath\n");
  fprintf (ps, "stroke\n");

  fprintf (ps, "newpath\n");
  fprintf (ps, "0 0 moveto\n");
  fprintf (ps, "0 %d lineto\n", mapHeight);
  fprintf (ps, "%d %d lineto\n", mapWidth, mapHeight);
  fprintf (ps, "%d 0 lineto\n", mapWidth);
  fprintf (ps, "0 0 lineto\n");
  fprintf (ps, "closepath\n");
  fprintf (ps, "clip\n");
}


void pstrailer()
{
  mapWidth = originalWidth;
  mapHeight = originalHeight;
  mapWidthcm = originalWidthcm;
  mapHeightcm = originalHeightcm;
  map_scale = originalScale;
  map_origin = originalorigin;

  fprintf (ps, "showpage\n");
  fprintf (ps, "quit\n");
}

void button_print()
{
  media = (device == 0) ? POSTSCRIPT: COLORPS;

  if (ps = fopen ("map.ps", "w"))
    {
      originalWidth = mapWidth;
      originalHeight = mapHeight;
      originalWidthcm = mapWidthcm;
      originalHeightcm = mapHeightcm;
      originalScale = map_scale;

      map_initcoord();

      psheader(1);

      fprintf (ps, "0.2 setlinewidth\n");
      fprintf (ps, "[] 0 setdash\n");
      coord_drawgrid (0);         /* draw coordinate grid */

      drawvisibleobjects (0);   /* objects other than CTRs... */
      drawvisibleobjects (1);   /* and CTRs */

      pstrailer();
     
      map_initcoord();

      fclose (ps);
    }
  media = SCREEN;
}
